/////////////////////////////////////////////////////////
// Actor.cpp - Implementation of the Actor Class
// Tylor Emmett
// GSP 281 - 10/3/14
/////////////////////////////////////////////////////////

#include "Actor.h"
#include "Building.h"
#include "ElevatorCar.h"
#include "Floor.h"
#include "Model.h"
#include "DataStorage.h"

void Actor::CallElevator()
{
	//Places actor in the queue for the floor they are currently on and records time. Does nothing if actor is in an elevator.
	if(Floor* castedLoc = dynamic_cast<Floor*>(m_Location))
	{
		//Get a pointer to the floor object
		Floor* floor = Building::GetInstance()->GetFloor(castedLoc->GetFloorNum());
		//Add this actor to the floor
		floor->AddActor(this);
		//Set the actor's location
		m_Location = (Location*)floor;
		//Change actor's queued state
		m_bQueued = true;
		//Data collection
		m_CurrentElevatorData.StartFloor = floor->GetFloorNum();
		m_CurrentElevatorData.StartTime = CURRENT_SIM_TIME;

	}
}

void Actor::BoardElevator(ElevatorCar* target)
{
	//Moves actor from floor to elevator and records time. Does nothing if in an elevator.
	if(Floor* castedLoc = dynamic_cast<Floor*>(m_Location))
	{
		//Remove this actor from their current floor
		Building::GetInstance()->GetFloor(castedLoc->GetFloorNum())->RemoveActor(this);
		//Add this actor to the elevator passed
		target->AddPassenger(this);
		//Add this actor's target floor
		target->AddStop(m_TargetFloor);
		//Set the actor's location
		m_Location = (Location*)target;
		//Change actor's queued state
		m_bQueued = false;
		//Data collection
		m_CurrentElevatorData.BoardTime = CURRENT_SIM_TIME;

	}
}

void Actor::DepartElevator()
{
	//Remove actor from elevator, collects remaining data and passes data to container.
	if(ElevatorCar* castedLoc = dynamic_cast<ElevatorCar*>(m_Location))
	{
		//Remove actor from elevator
		castedLoc->RemovePassenger(this);
		//Set the actor's location
		m_Location = (Location*)Building::GetInstance()->GetFloor(castedLoc->GetCurrentFloor());
		//Data collection
		m_CurrentElevatorData.DepartTime = CURRENT_SIM_TIME;
		m_CurrentElevatorData.EndFloor = castedLoc->GetCurrentFloor();
		//Pass data on to storage
		DataStorage::GetInstance()->AddData(&m_CurrentElevatorData);
	}
}

Actor::Actor(__int64 ID, __int64 targetFloor):m_ID(ID), m_TargetFloor(targetFloor)
{
	//Set location to Ground Floor.
	auto startLocation = Building::GetInstance()->GetFloor(1);
	startLocation->AddActor(this);
	m_Location = (Location*)startLocation;
	m_bQueued = false;
}

__int64 Actor::GetID()
{
	return m_ID;
}

__int64 Actor::GetTargetFloor()
{
	return m_TargetFloor;
}