/////////////////////////////////////////////////////////
// ActorFactory.cpp - Implementation of the ActorFactory class
// Tylor Emmett
// GSP 281 - 10/1/14
/////////////////////////////////////////////////////////

#include "ActorFactory.h"
#include "ActorManager.h"
#include <random>
#include <iostream>
#include "Employee.h"
#include "Model.h"
#include "Customer.h"

void ActorFactory::Update(float dt)
{
	//Find the current time
	float currentTime = Model::GetInstance()->GetSimulationTime();


	//If between 7:30 and 8:00, spawn employees according to Poisson rate created in c-tor
	if((currentTime >= 27000.0f) && (currentTime < 28800.0f))
	{
		int currentMinute = ((int)currentTime - 27000) / 60;
		if(m_EmployeeCreationRate[currentMinute] > 0)
		{
			//Create however many employees should have been created this minute. Note: I attempted to distribute them across each minute, but speeding up the time caused some employees to be skipped.
			for(;m_EmployeeCreationRate[currentMinute] > 0; m_EmployeeCreationRate[currentMinute]--)
				if(m_NextValidID <= 400) //To keep rounding issues from creating 1 too many, or 1 too few.
					CreateEmployee(false, 5);	//Spawn an employee
		}
	}
	//If between 8:00 and 5:00 spawn customers at a rate of 1 per 5 minutes.
	if((currentTime >= 28800.0f) && (currentTime < 61200.0f))
	{
		m_CustomerCreateTime -= dt;
		if(m_CustomerCreateTime <= 0.f)
		{
			std::uniform_int_distribution<__int64> dist(1,5);
			CreateCustomer(dist(mt));
			m_CustomerCreateTime = 300.f;
		}
	}
}

ActorFactory* ActorFactory::GetInstance()
{
	static ActorFactory instance;
	return &instance;
}

void ActorFactory::CreateEmployee(bool lunch, __int64 floor)
{
	Employee* nextEmployee = new Employee(lunch, m_NextValidID, floor);
	ActorManager::GetInstance()->AddEmployee(nextEmployee);
	m_NextValidID++;
}

void ActorFactory::CreateCustomer(__int64 floor)
{
	std::uniform_int_distribution<__int64> dist(900, 2700);
	Customer* nextCustomer = new Customer(dist(mt), m_NextValidID, floor);
	ActorManager::GetInstance()->AddCustomer(nextCustomer);
	m_NextValidID++;
}
	
ActorFactory::ActorFactory()
{
	m_NextValidID = 1;
	std::random_device rd;
	mt.seed(rd());

	//Init random number generation
	std::uniform_int_distribution<__int64> dist(0, 399); //0-399 will be the IDs of the 400 employees, as they are created first.
	std::poisson_distribution<> pDist(16);

	//Initialize the lunch list. Creates a list of IDs that will be leaving for lunch. Upon creation, any agents with these IDs will be tagged as such.
	while(m_LunchEmployees.size() <= 200)
	{
		//Generate a random number
		__int64 nextID = dist(mt);
		//Check the vector for that ID
		std::vector<__int64>::iterator itr;
		itr = std::find(m_LunchEmployees.begin(), m_LunchEmployees.end(), nextID);
		//If it is not listed already, add it to the vector.
		if(itr == m_LunchEmployees.end())
		{
			m_LunchEmployees.push_back(nextID);
		}

	}

	//Creates a 400 employee poisson arrival rate. Stores the number of arrivals per minute from 7:30 to 7:59.
	for(unsigned i = 0; i <= 400; i++)
	{
		int number = pDist(mt);
		if(number < 30)
			++m_EmployeeCreationRate[number];

	}

	m_CustomerCreateTime = 300.f;
}

