/////////////////////////////////////////////////////////
// ActorFactory.h - Declaration of the ActorFactory class
// Tylor Emmett
// GSP 281 - 10/1/14
/////////////////////////////////////////////////////////

#pragma once
#include <vector>
#include <random>

class ActorFactory
{
	__int64 m_NextValidID;								//Next ID that may be assigned to an actor
	std::vector<__int64> m_LunchEmployees;				//IDs of the employees who will be sent to lunch when created.
	std::vector<__int64> m_FloorAssigments;				//Number of employees assigned to each floor. Used to keep from overflowing a floor.
	int m_EmployeeCreationRate[30];						//Number of employees created per minute from 7:30 to 7:59 (30 minutes).
	std::mt19937 mt;									//Engine for mersenne twister random numbers.
	float m_CustomerCreateTime;							//Time until next customer can be created.
public:

	void Update(float dt);								//Updates the factory
	static ActorFactory* GetInstance();					//Accessor for singleton class

private:
	void CreateEmployee(bool lunch, __int64 floor);		//Adds a new employee to the simulation
	void CreateCustomer(__int64 floor);					//Adds a new customer to the simulation
	//Private c-tors and assignment operator to prevent multiple objects of this type.
	ActorFactory();
	ActorFactory(ActorFactory&){}
	void operator=(ActorFactory&){}
};