/////////////////////////////////////////////////////////
// ActorManager.cpp - Implementation of the ActorManager class.
// Tylor Emmett
// GSP 281 - 10/1/14
/////////////////////////////////////////////////////////

#include "Customer.h"
#include "Employee.h"
#include "ActorManager.h"
#include <iostream>

void ActorManager::AddCustomer(Customer* customer)
{
	//Check to see if the customer doesn't already exist. Add if not.
	if(m_CustomerMap.find(customer->GetID()) == m_CustomerMap.end())
		m_CustomerMap[customer->GetID()] = customer;
}

void ActorManager::AddEmployee(Employee* employee)
{
	//Check to see if the employee doesn't already exist. Add if not.
	if(m_EmployeeMap.find(employee->GetID()) == m_EmployeeMap.end())
		m_EmployeeMap[employee->GetID()] = employee;
}

void ActorManager::RemoveActor(__int64 ID)
{
	//Check for the ID in the customer list. Remove if found.
	if(m_CustomerMap.find(ID) != m_CustomerMap.end())
		m_CustomerMap.erase(ID);
	//If not in customer list, check employee list. Remove if found.
	else if(m_EmployeeMap.find(ID) != m_EmployeeMap.end())
		m_EmployeeMap.erase(ID);
}

Actor* ActorManager::GetActor(__int64 ID)
{
	//Check for the ID in the customer list. Return if found.
	if(m_CustomerMap.find(ID) != m_CustomerMap.end())
		return m_CustomerMap[ID];
	//If not in customer list, check employee list. Return if found.
	else if(m_EmployeeMap.find(ID) != m_EmployeeMap.end())
		return m_EmployeeMap[ID];
	else
		return nullptr;
}

__int64 ActorManager::GetEmployeeCount(__int64 floorNum)
{
	return (__int64)m_EmployeeMap.size();
}

__int64 ActorManager::GetCustomerCount(__int64 floorNum)
{
	return (__int64)m_CustomerMap.size();
}

void ActorManager::Update(float dt)
{
	//Setup iterators
	std::map<__int64, Customer*>::iterator customerItr;
	std::map<__int64, Employee*>::iterator employeeItr;
	//Update all customers
	for(customerItr = m_CustomerMap.begin(); customerItr != m_CustomerMap.end(); customerItr++)
	{
		customerItr->second->Update(dt);
	}
	//Update all employees
	for(employeeItr = m_EmployeeMap.begin(); employeeItr != m_EmployeeMap.end(); employeeItr++)
	{
		employeeItr->second->Update(dt);
	}
}

ActorManager* ActorManager::GetInstance()
{
	static ActorManager instance;
	return &instance;
}

void ActorManager::Render()
{
	std::cout << "Actors In System: " << (m_CustomerMap.size() + m_EmployeeMap.size()) << std::endl;
}