/////////////////////////////////////////////////////////
// ActorManager.h - Declaration of the ActorManager class.
// Tylor Emmett
// GSP 281 - 10/1/14
/////////////////////////////////////////////////////////

#pragma once

#include <map>
#include <vector>

//Forward Declarations
class Customer;
class Employee; 
class Actor;

class ActorManager
{
	std::map<__int64, Customer*> m_CustomerMap;	//Stores a pointer to all customers in the simulation
	std::map<__int64, Employee*> m_EmployeeMap;	//Stores a pointer to all employees in the simulation
	std::vector<__int64> m_LunchEmployees;		//Stores a list of employees that will be leaving for lunch.
public:
	void AddCustomer(Customer* customer);		//Adds a customer to the cache
	void AddEmployee(Employee* employee);		//Adds an employee to the cache
	void RemoveActor(__int64 ID);				//Removes an actor based on ID
	Actor* GetActor(__int64 ID);				//Returns a pointer to an actor based on ID
	__int64 GetEmployeeCount(__int64 floorNum);	//Returns the number of employees currently on a floor.
	__int64 GetCustomerCount(__int64 floorNum); //Returns the number of customers currently on a floor.
	void Update(float dt);						//Update all customers and employees
	static ActorManager* GetInstance();			//Singleton accessor
	void Render();								//Outputs status to screen.
private:
	//Private c-tors and assignment operator to prevent multiple objects of this type.
	ActorManager(){}
	ActorManager(ActorManager&){}
	void operator=(ActorManager&){}

};