/////////////////////////////////////////////////////////
// Builder.cpp - Implementation of the Building class.
// Tylor Emmett
// GSP 281 - 10/1/14
/////////////////////////////////////////////////////////

#include "Building.h"
#include "ElevatorCar.h"
#include "Floor.h"
#include <algorithm>



void Building::Update(float dt)
{
	std::vector<__int64> upFloors, downFloors, idleElevators, upElevators, downElevators;
	//Check all floors for queues.
	auto floorItr = m_Floors.begin();
	for(;floorItr != m_Floors.end(); floorItr++)
	{
		Floor* currentFloor = *floorItr;
		if(currentFloor->HasUpQueue())
			upFloors.push_back(currentFloor->GetFloorNum());
		if(currentFloor->HasDownQueue())
			downFloors.push_back(currentFloor->GetFloorNum());
	}
	//Check elevator status.
	for(unsigned i = 0; i < m_Elevators.size(); i++)
	{
		//Find idle elevators
		if(m_Elevators[i]->GetState() == Idle)
		{
			idleElevators.push_back(i);
			continue;
		}
		//Find moving elevators
		else if((m_Elevators[i]->GetState() == Moving) && (m_Elevators[i]->GetDirection() == Up))
		{
			upElevators.push_back(i);
		}
		else if((m_Elevators[i]->GetState() == Moving) && (m_Elevators[i]->GetDirection() == Down))
		{
			downElevators.push_back(i);
		}
	}

	//Sort target arrays
	std::sort(upFloors.begin(), upFloors.end());
	std::sort(downFloors.begin(), downFloors.end());
	//If one elevator is idle, send it in the direction with the most floor targets
	if(idleElevators.size() == 1)
	{
		//If more up floors
		if(upFloors.size() > downFloors.size())
		{
			//Add all stops at or above this elevator
			for(unsigned i = 0; i < upFloors.size(); i++)
			{
				if(upFloors[i] >= m_Elevators[idleElevators[0]]->GetCurrentFloor())
				{
					m_Elevators[idleElevators[0]]->AddStop(upFloors[i]);
				}
			}
		}

		//If more down floors
		if(upFloors.size() < downFloors.size())
		{
			//Add all stops at or below this elevator
			for(unsigned i = 0; i < downFloors.size(); i++)
			{
				if(downFloors[i] <= m_Elevators[idleElevators[0]]->GetCurrentFloor())
				{
					m_Elevators[idleElevators[0]]->AddStop(downFloors[i]);
				}
			}
		}
	}
	//If 2 or more are idle, send one in each direction (if there are queues in both directions)
	else if(idleElevators.size() >= 2)
	{
		//Add all stops at or below the first elevator
		for(unsigned i = 0; i < downFloors.size(); i++)
		{
			if(downFloors[i] <= m_Elevators[idleElevators[0]]->GetCurrentFloor())
			{
				m_Elevators[idleElevators[0]]->AddStop(downFloors[i]);
			}
		}

		//Add all stops at or above the second elevator
		for(unsigned i = 0; i < upFloors.size(); i++)
		{
			if(upFloors[i] >= m_Elevators[idleElevators[1]]->GetCurrentFloor())
			{
				m_Elevators[idleElevators[1]]->AddStop(upFloors[i]);
			}
		}
	}
	//If no idle elevators, check the moving ones
	else
	{
		//If there are down queues, check the elevators moving down
		if(downFloors.size() > 0)
		{
			for(unsigned i = 0; i < downElevators.size(); i++)
			{
				//Add all stops below this elevator
				for(unsigned i = 0; i < downFloors.size(); i++)
				{
					if(downFloors[i] < m_Elevators[downElevators[i]]->GetCurrentFloor())
					{
						m_Elevators[downElevators[i]]->AddStop(downFloors[i]);
					}
				}
			}
		}
		//If there are Up queues, check the elevators moving up
		if(upFloors.size() >0)
		{
			for(unsigned i = 0; i < upElevators.size(); i++)
			{
				//Add all stops below this elevator
				for(unsigned i = 0; i < upFloors.size(); i++)
				{
					if(upFloors[i] < m_Elevators[upElevators[i]]->GetCurrentFloor())
					{
						m_Elevators[upElevators[i]]->AddStop(upFloors[i]);
					}
				}
			}
		}
	}

	//Update elevators
	for(unsigned i = 0; i < m_Elevators.size(); i++)
	{
		m_Elevators[i]->Update(dt);
	}

}

Building* Building::GetInstance()
{
	static Building instance;
	return &instance;
}

void Building::Init(__int64 numFloors, __int64 numElevators)
{
	//Create floors
	if(numFloors > 0)
	{
		//Create ground floor
		GroundFloor* ground = new GroundFloor;
		//Add to list
		m_Floors.push_back(ground);
		//Create remaining floors
		for(__int64 i = 2; i <= numFloors; ++i)
		{
			Floor* nextFloor = new Floor(i);
			m_Floors.push_back(nextFloor);
		}
	}

	//Create elevators
	if(numElevators > 0)
	{
		for(__int64 i = 0; i < numElevators; ++i)
		{
			ElevatorCar* nextElevator = new ElevatorCar(i+1);
			m_Elevators.push_back(nextElevator);
		}
	}
}

Floor* Building::GetFloor(__int64 floorNum)
{
	if(m_Floors.size() >= (floorNum - 1))
		return m_Floors[floorNum - 1];
	else
		return nullptr;
}

void Building::Render()
{
	auto elevators = m_Elevators.begin();
	auto floors = m_Floors.begin();
	for(;elevators != m_Elevators.end(); elevators++)
	{
		ElevatorCar* currentElevator = *elevators;
		currentElevator->Render();
	}

	for(;floors != m_Floors.end(); floors++)
	{
		Floor* currentFloor = *floors;
		currentFloor->Render();
	}

}

unsigned Building::GetElevatorCount()
{
	return m_Elevators.size();
}