/////////////////////////////////////////////////////////
// ElevatorCar.cpp - Implementation of the ElevatorCar class.
// Tylor Emmett
// GSP 281 - 10/3/14
/////////////////////////////////////////////////////////

#include "ElevatorCar.h"
#include "Actor.h"
#include "Building.h"
#include "Floor.h"
#include <algorithm>
#include <iostream>


void ElevatorCar::Update(float dt)
{
	//Change directions at the top and bottom of building.
	if(m_CurrentFloor == 5)
		m_CurrentDirection = Down;
	else if(m_CurrentFloor == 1)
		m_CurrentDirection = Up;

	switch(m_CurrentState)
	{
	case Idle:
		//Check for a new target floor.
		if(m_TargetFloors.size() != 0)
		{
			//If below current target, change state to moving, and direction to downward
			if(m_TargetFloors[0] < m_CurrentFloor)
			{
				m_CurrentState = Moving;
				m_CurrentDirection = Down;
			}
			//If above current target, change state to moving, and direction to upward
			else if(m_TargetFloors[0] > m_CurrentFloor)
			{
				m_CurrentState = Moving;
				m_CurrentDirection = Up;
			}
			//If equal to current target, change state to loading.
			else if(m_TargetFloors[0] == m_CurrentFloor)
			{
				m_CurrentState = Loading;
			}
		}
		break;
	case Moving:
		//Check if floor reached.
		if(m_TimeToNextFloor <= 0.0f)
		{
			//If moving up, add 1 to current floor. Subtract 1 if moving down.
			if(m_CurrentDirection == Up)
			{
				if(m_CurrentFloor < 5)
					m_CurrentFloor++;
			}
			else
			{
				if(m_CurrentFloor > 1)
					m_CurrentFloor--;
			}
			//If this floor is a target, change to unloading
			auto itr = std::find(m_TargetFloors.begin(), m_TargetFloors.end(), m_CurrentFloor);
			if(itr != m_TargetFloors.end())
				m_CurrentState = Unloading;
			//Reset floor travel timer
			m_TimeToNextFloor = m_FloorTravelTime;
		}
		else
		{
			//If not, continue moving.
			m_TimeToNextFloor -= dt;
		}
		break;
	case Loading:
		//Try to load a passenger if there is room
		if(m_Passengers.size() < m_Capacity)
		{
			if((Building::GetInstance()->GetFloor(m_CurrentFloor)->HasUpQueue()) || (Building::GetInstance()->GetFloor(m_CurrentFloor)->HasDownQueue()))
			{
				Building::GetInstance()->GetFloor(m_CurrentFloor)->LoadPassenger(this);
			}
			//If no passengers to load, remove this floor from the targets.
			else
			{
				if(m_TargetFloors.size() > 0)
					m_TargetFloors.erase(m_TargetFloors.begin());
			}
		}
		//Change state to moving if full.
		else
		{
			m_CurrentState = Moving;
		}
		break;
	case Unloading:
		if(m_Passengers.size() > 0)
		{
			//Try to unload a passenger
			for(unsigned i = 0; i < m_Passengers.size(); i++)
			{
				if(m_Passengers[i]->GetTargetFloor() == m_CurrentFloor)
				{
					m_Passengers[i]->DepartElevator();
					return;
				}
			}
			//Change state to loading to see if passengers need to get on.
			m_CurrentState = Loading;
			
		}
		//Clear targets if no passengers remain
		else
		{
			m_TargetFloors.clear();
			m_CurrentState = Idle;
		} 
		break;
	}
}

void ElevatorCar::AddStop(__int64 floorNumber)
{
	//Add stop if not already in list
	if(std::find(m_TargetFloors.begin(), m_TargetFloors.end(), floorNumber) == m_TargetFloors.end())
	{
		m_TargetFloors.push_back(floorNumber);
	}
	//Sort the array
	std::sort(m_TargetFloors.begin(), m_TargetFloors.end());
}

void ElevatorCar::AddPassenger(Actor* actor)
{
	//Add agent if not in list
	auto itr = std::find(m_Passengers.begin(), m_Passengers.end(), actor);
	if(itr == m_Passengers.end())
		m_Passengers.push_back(actor);
}

void ElevatorCar::RemovePassenger(Actor* actor)
{
	//If actor is in list, remove it.
	auto itr = std::find(m_Passengers.begin(), m_Passengers.end(), actor);
	if(itr != m_Passengers.end())
		m_Passengers.erase(itr);
}

ElevatorCar::ElevatorCar(__int64 ID):m_ID(ID)
{
	m_CurrentState = Idle;
	m_CurrentDirection = Up;
	m_CurrentFloor = 1;
	m_TimeToNextFloor = m_FloorTravelTime;
	
}

void ElevatorCar::UnloadPassenger()
{
	//Unload the first passenger with this floor as a target.
	for(unsigned i = 0; i < m_Passengers.size(); i++)
	{
		if(m_Passengers[i]->GetTargetFloor() == m_CurrentFloor)
		{
			m_Passengers[i]->DepartElevator();
			break;
		}
	}
}

void ElevatorCar::LoadPassenger()
{
	Floor* currentFloor = Building::GetInstance()->GetFloor(m_CurrentFloor);
	//Lets the floor object know to load a passenger.
	currentFloor->LoadPassenger(this);
}

__int64 ElevatorCar::GetCurrentFloor()
{
	return m_CurrentFloor;
}

ElevatorDirection ElevatorCar::GetDirection()
{
	return m_CurrentDirection;
}

ElevatorState ElevatorCar::GetState()
{
	return m_CurrentState;
}

void ElevatorCar::Render()
{
	//Output current status
	
	std::cout << "Elevator " << m_ID << " State: ";
	switch(m_CurrentState)
	{
	case Idle:
		std::cout << "Idle ";
		break;
	case Moving:
		std::cout << "Moving ";
		break;
	case Loading:
		std::cout << "Loading ";
		break;
	case Unloading:
		std::cout << "Unloading ";
		break;
	default:
		std::cout << "InvalidState ";
	}
	std::cout << "Direction: ";
	if(m_CurrentDirection == Up)
		std::cout << "Up ";
	else
		std::cout << "Down ";
	std::cout << "Passengers: " << m_Passengers.size() << " Current Floor: " << m_CurrentFloor << std::endl;
}

const float ElevatorCar::m_FloorTravelTime = 15.0f;
const float ElevatorCar::m_DoorTime = 10.0f;
const float ElevatorCar::m_LoadDelay = 3.0f;
const __int64 ElevatorCar::m_Capacity = 8;