/////////////////////////////////////////////////////////
// ElevatorCar.h - Declaration of the ElevatorCar class.
// Tylor Emmett
// GSP 281 - 10/1/14
/////////////////////////////////////////////////////////

#pragma once

#include <vector>
#include "Location.h"

class Actor;

enum ElevatorState{Moving, Loading, Unloading, Idle};	//Potential Elevator states
enum ElevatorDirection{Up, Down};						//Potential elevator directions

class ElevatorCar : public Location
{
	ElevatorState m_CurrentState;							//Current Elevator state
	ElevatorDirection m_CurrentDirection;					//Current Elevator direction
	
	std::vector<Actor*> m_Passengers;						//List of passengers
	std::vector<__int64> m_TargetFloors;					//List of floors the elevator needs to stop at
	__int64 m_CurrentFloor;									//Floor the elevator is or was (if moving) at.
	float m_TimeToNextFloor;								//Time remaining to reach next floor
	static const float m_FloorTravelTime;					//Time required between floors
	static const __int64 m_Capacity;						//Max number of passengers
	static const float m_LoadDelay;							//Time between loading and unloading passengers
	static const float m_DoorTime;							//Time to close doors after loading.
	__int64 m_ID;											//Elevator ID

public:

	void Update(float dt);									//Update elevator
	void AddStop(__int64 floorNumber);						//Add a floor to the list of stops
	void AddPassenger(Actor* actor);						//Add a passenger to the list
	void RemovePassenger(Actor* actor);						//Remove a passenger from the list
	ElevatorCar(__int64 ID);								//Parameterized c-tor
	__int64 GetCurrentFloor();								//Returns the current floor number.
	ElevatorState GetState();								//Returns current elevator state
	ElevatorDirection GetDirection();						//Returns direction
	void Render();											//Outputs data to screen.

private:
	
	void UnloadPassenger();
	void LoadPassenger();
	ElevatorCar(){}											//Private default c-tor to force parameterized c-tor.
};