/////////////////////////////////////////////////////////
// Floor.cpp - Implementation of the Floor class.
// Tylor Emmett
// GSP 281 - 10/1/14
/////////////////////////////////////////////////////////

#include "Floor.h"
#include "Employee.h"
#include "Customer.h"
#include "ElevatorCar.h"
#include <algorithm>
#include <iostream>

void Floor::AddActor(Actor* actor)
{
	//If the actor is not in one of the queues, add it based on direction of travel.
	auto upItr = std::find(m_UpQueue.begin(), m_UpQueue.end(), actor);
	auto downItr = std::find(m_DownQueue.begin(), m_DownQueue.end(), actor);
	if((upItr == m_UpQueue.end()) && (downItr == m_DownQueue.end()))
	{
		if(actor->GetTargetFloor() > m_FloorNumber)
			m_UpQueue.push_back(actor);
		else if (actor->GetTargetFloor() < m_FloorNumber)
			m_DownQueue.push_back(actor);
	}
}

void Floor::RemoveActor(Actor* actor)
{
	//If actor is in one of the queues, remove it.
	auto upItr = std::find(m_UpQueue.begin(), m_UpQueue.end(), actor);
	auto downItr = std::find(m_DownQueue.begin(), m_DownQueue.end(), actor);
	//If in Up queue and not in Down queue, remove from up queue.
	if((upItr != m_UpQueue.end()) && (downItr == m_DownQueue.end()))
		m_UpQueue.erase(upItr);
	//If in Down queue and not in up queue, remove from down queue.
	else if((upItr == m_UpQueue.end()) && (downItr != m_DownQueue.end()))
		m_DownQueue.erase(downItr);
	
}
	
Floor::Floor(__int64 floorNum):m_FloorNumber(floorNum)
{
	
}

void Floor::LoadPassenger(ElevatorCar* elevator)
{
	//Confirm elevator state
	if(elevator->GetState() == Loading)
	{
		//Check elevator direction
		if(elevator->GetDirection() == Up)
		{
			if(m_UpQueue.size() > 0)
			{
				auto nextPassenger = m_UpQueue[0];
				nextPassenger->BoardElevator(elevator);
			}
		}
		else if (elevator->GetDirection() == Down)
		{
			if(m_DownQueue.size() > 0)
			{
				auto nextPassenger = m_DownQueue[0];
				nextPassenger->BoardElevator(elevator);
			}
		}
	}
}

__int64 Floor::GetFloorNum()
{
	return m_FloorNumber;
}

GroundFloor::GroundFloor():Floor(1)
{

}

void Floor::Render()
{
	//Output current status
	std::cout << "Floor " << m_FloorNumber << " Up: " << m_UpQueue.size() << " Down: " << m_DownQueue.size() << std::endl;
}

bool Floor::HasDownQueue()
{
	return m_UpQueue.size() > 0 ? true : false;
}

bool Floor::HasUpQueue()
{
	return m_DownQueue.size() > 0 ? true : false;
}
