/////////////////////////////////////////////////////////
// Model.cpp - Implementation of the Model Class.
// Tylor Emmett
// GSP 281 - 10/1/14
/////////////////////////////////////////////////////////

#include "Model.h"
#include "windows.h"
#include <iostream>
#include "ActorFactory.h"
#include "Building.h"
#include "ActorManager.h"
#include "DataStorage.h"

Model::Model()
{


}

void Model::BeginSimulation()
{
	//Init frame time.
	m_StartingCount = 0;
	m_CurrentCount = 0;

	__int64 cntsPerSec = 0;
	QueryPerformanceFrequency((LARGE_INTEGER*)&cntsPerSec);
	m_SecondsPerCount = 1.0f / (float)cntsPerSec;
		
	__int64 prevTimeStamp = 0;
	QueryPerformanceCounter((LARGE_INTEGER*)&prevTimeStamp);
	m_StartingCount = prevTimeStamp;

	//Set simulation state
	m_bSimulationRunning = true;

	//Initialize subsystems and objects
	ActorFactory::GetInstance();


	//Get number of elevators
	__int64 numElevators;
	std::cout << "Please input number of elevators.";
	std::cin >> numElevators;

	Building::GetInstance()->Init(5, numElevators);

	//Primary loop
	while(m_bSimulationRunning)
	{
		//Update frame time.
		QueryPerformanceCounter((LARGE_INTEGER*)&m_CurrentCount);
		
		float dt = (m_CurrentCount - prevTimeStamp)*m_SecondsPerCount * 20;
		std::cout << "Delta Time: " << dt << std::endl;
		//Update objects
		Update(dt);

		//Render/output objects.
		Render();

		//Update prev time stamp
		prevTimeStamp = m_CurrentCount;
	}
	DataStorage::GetInstance()->ProcessData();
}


void Model::EndSimulation()
{
	//Set simulation state back to fasle.
	m_bSimulationRunning = false;
}


void Model::Update(float dt)
{
	ActorFactory::GetInstance()->Update(dt);
	Building::GetInstance()->Update(dt);
	if(GetSimulationTime() > END_TIME_SECONDS)
		EndSimulation();
}


void Model::Render()
{
	system("cls");
	ShowSimulationTime();
	Building::GetInstance()->Render();
	ActorManager::GetInstance()->Render();

}

float Model::GetSimulationTime()
{
	return (m_CurrentCount  - m_StartingCount) * m_SecondsPerCount * 20 + START_TIME_SECONDS;
}

void Model::ShowSimulationTime()
{
	__int64 hours, minutes, seconds;

	//Converting total time in seconds to a 24 hour format, with days leading
	seconds = (__int64)GetSimulationTime();
	minutes = seconds / 60;
	seconds = seconds % 60;
	hours = minutes / 60;
	minutes = minutes % 60;
	hours = hours % 24;

	std::cout << "Sim Time: ";
	if(hours < 10)
		std::cout << "0" << hours;
	else
		std::cout << hours;
	std::cout << ":";
	if(minutes < 10)
		std::cout << "0" << minutes;
	else
		std::cout << minutes;
	std::cout << ":" << seconds << std::endl;
}

Model* Model::GetInstance()
{
	static Model instance;
	return &instance;
}